@@ -99,12 +99,14 @@ parseconf_bool_array[] =
   { "ssl_request_cert", &tunable_ssl_request_cert },
   { "delete_failed_uploads", &tunable_delete_failed_uploads },
   { "implicit_ssl", &tunable_implicit_ssl },
-  { "sandbox", &tunable_sandbox },
+  { "ptrace_sandbox", &tunable_ptrace_sandbox },
   { "require_ssl_reuse", &tunable_require_ssl_reuse },
   { "isolate", &tunable_isolate },
   { "isolate_network", &tunable_isolate_network },
   { "ftp_enable", &tunable_ftp_enable },
   { "http_enable", &tunable_http_enable },
+  { "seccomp_sandbox", &tunable_seccomp_sandbox },
+  { "allow_writeable_chroot", &tunable_allow_writeable_chroot },
   { 0, 0 }
 };
 
